#ifndef _CHAMPSSCALAIRES_H
#define _CHAMPSSCALAIRES_H

#include <iostream>

/*
 Title: Module pour la gestion des champs scalaires
 */


/*
 Class: ChampsScalaires
  Permet la gestion de champs de scalaires de type float
*/
class ChampsScalaires {

 private:
  float * tab;
  int taillex;
  int tailley;



 public:
  /*
   Constructor: ChampsScalaires
    Paramtres:
     n - taille horizontale du champs de scalaires
     m - taille verticale du champs de scalaires
  */
  ChampsScalaires(int n, int m);

  /*
   Destructor: ~ChampsScalaires
   */
   ~ChampsScalaires();

  /*
   Method: ecrire
    Permet d'crire un float  une certaine position

    Paramtres:
       i - position horizontale
       j - position verticale
       element - le flottant  mettre

    Prconditions:
      i doit tre entre 0 et n-1 et j doit tre entre 0 et m-1
  */
  void ecrire(int i, int j, float element) { tab[i + taillex * j] = element;}

 /*
  Method: lire
   Permet la lecture d'un lment

   Paramtres:
      i - position horizontale
       j - position verticale

   Retour:
    Le flottant se trouvant  ces positions

  Prconditions:
      i doit tre entre 0 et n-1 et j doit tre entre 0 et m-1
  */
  float lire(int i, int j) const {
      return tab[i+taillex *j];}

  /*
   Method: max
    Retourne le maximum de tous les flottants de la matrice
  */
  float max() const;

  /*
   Method: min
    Retourne le minimum de tous les flottants du champs
  */
  float min() const;

  /*
   Method: annuler
     Annule le champs de scalaires
  */
  void annuler();

  /*
   Method: tailleX
   Retour: Retourne la taille horizontale du champs
  */
  int tailleX() const;

  /*
    Method: tailleY
    Retour: Retourne la taille verticale du champs
  */
  int tailleY() const;

  /*
   Method: somme
    Ajoute un lment au champs scalaires

    Paramtres:
     i - position horizontale
     j - position verticale
     element - element  ajouter

  */
  void somme(int i, int j, float element);

  /*
    Method: clone
     Construit une clone du champs de scalaires
  */
  ChampsScalaires* clone() const;

  /*
   Method: copie
    Copie dans un autre champs de scalaires

    Prconditions:
     Les deux champs scalaires doivent avoir la mme taille
  */
  void copie(ChampsScalaires*)  const;

  /*
   Method: pas
     Retourne le pas de discrtisation spatiale du champs
  */
  float pas() const;

  void sommeScalaires(ChampsScalaires* c, float coeff);
};



#endif
