#ifndef _CHAMPSVECTEURS_H
#define _CHAMPSVECTEURS_H

#include "champsscalaires.h"

/*
 Title: Module de gestion de champs de vecteurs
 */

/* Topic:

*/

/*
 Class: ChampsVecteurs
   Type abstrait permettant la gestion de champs de vecteurs deux dimensions de flottant
*/
class ChampsVecteurs {
 private:
  ChampsScalaires *x;
  ChampsScalaires *y;

 public:
 /*
  Constructor: ChampsVecteurs
  Paramtres:
   n - taille horizontale du champs de vecteurs
   m - taille verticale
  */
  ChampsVecteurs(int n, int m);

  ~ChampsVecteurs();


  /*
   Method: getX
   Retour: Retourne le champs de scalaires associe  la projection sur la droite X
  */
  ChampsScalaires* getX()  const;

  /*
   Method: getY
   Retour: Retourne le champs de scalaires associe  la projection sur la droite Y
  */
  ChampsScalaires* getY()  const;

  /*
   Method: tailleX
   Retour: Retourne la taille horizontale du champs
  */
  int tailleX()  const;

  /*
   Method: tailleY
   Retour: Retourne la taille verticale du champs
  */
  int tailleY()  const;


  void ecrire(int i, int j, float ex, float ey);

  /*
    Method: pas
    Retour: Retourne le pas de discrtisation spatiale du champs
  */
  float pas()  const;

  /*
   Method: clone
   Retour: Permet de retourner un clone du champs de vecteurs
  */ChampsVecteurs* clone()  const;

  /*
   Method: copie
    Permet la copie dans un autre champs de vecteurs

    Prconditions: c doit tre exactement de la mme taille que l'objet appelant
  */
  void copie(ChampsVecteurs* c)  const;

  /*
   Method: annuler
    Annule le champs de vecteurs
  */
  void annuler();

};


#endif
