#include <iostream>
#include <cmath>

#include "champsvecteurs.h"
#include "champsscalaires.h"
#include "navier.h"

#include "graphique.h"


#include <GL/glut.h>

typedef enum
{
  VITESSE, VORTEX, DIFFUSION, PRESSION
} WhatDraw;


using namespace std;
static WhatDraw whatdraw = VITESSE;

static int win_x = 800;
static int win_y = 600;

static Fluide* fluide_stat;

//static bool whatiwant = true;


static int mx;
static int my;
static int omx;
static int omy;
static int mouse_down[3];
static float forces;
static int win_id;

inline float maxfloat(float a, float b)
{
 if(a<b)
  return b; else return a;
}

inline float minfloat(float a, float b)
{
 if(a<b)
  return a; else return b;
}

static void dessineVitesse ()
{
	int i, j;
	float x, y;

  std::pair<float, float> vectvitesse;

  int N = fluide_stat->tailleX()-2;
  int M = fluide_stat->tailleY()-2;

	float hx = 1.0f/N;
	float hy = 1.0f/M;
  float vx;
  float vy;
  float norme;


	glColor3f ( 1.0f, 1.0f, 1.0f );
	glLineWidth ( 1.0f );

	glBegin ( GL_LINES );

		for ( i=1 ; i<=N ; i++ ) {
			x = (i-0.5f) * hx ;
			for ( j=1 ; j<=M ; j++ ) {
				y = (j-0.5f) *hy;

        vectvitesse = fluide_stat->vitesseLire(i,j);
        vx = vectvitesse.first;
        vy = vectvitesse.second;


        norme = 10 * sqrt(vx * vx + vy * vy);
      //  glColor3f (2*norme , 0.5 * norme, 2*norme );
			//  glColor3f( 0.0f, 78.0f / 255.0f, 255.0f / 255.0f);
			 glColor3f(0.0f,0.33f, 1.0f);
				glVertex2f ( x, y );
				glVertex2f ( x+  vx/2.0f, y+vy/2.0f );

      //  glColor3f (norme , 0.5f * norme, 2*norme );
//			  glColor3f( 0.0f, 255.0f / 255.0f, 84.0f / 255.0f);
     glColor3d(0.0,1.0f, 0.33f);
        glVertex2f ( x+  vx/2.0f, y+vy/2.0f );
				glVertex2f ( x+  vx, y+vy);


      }
		}

	glEnd ();
}


static void dessineDiffusion ()
{
	int i, j;
	float x, y, d00, d01, d10, d11;

  int N = fluide_stat->tailleX()-2;
  int M = fluide_stat->tailleY()-2;
	float hx = 1.0f/N;
	float hy = 1.0f/M;
	glBegin ( GL_QUADS );

		for ( i=0 ; i<=N ; i++ ) {
			x = (i-0.5f)*hx;
			for ( j=0 ; j<=M ; j++ ) {
				y = (j-0.5f)*hy;

				d00 = fluide_stat->diffusionLire(i,j);
				d01 = fluide_stat->diffusionLire(i,j+1);
				d10 = fluide_stat->diffusionLire(i+1,j);
				d11 = fluide_stat->diffusionLire(i+1,j+1);

				glColor3f ( d00, d00/2, d00/2 ); glVertex2f ( x, y );
				glColor3f ( d10, d10/2, d10/2 ); glVertex2f ( x+hx, y );
				glColor3f ( d11, d11/2, d11/2 ); glVertex2f ( x+hx, y+hy );
				glColor3f ( d01, d01/2, d01/2 ); glVertex2f ( x, y+hy );
			}
		}

	glEnd ();
}

static void dessinePression ()
{
	int i, j;
  float x, y, d00, d01, d10, d11, d00r, d00g, d01r, d01g, d10r, d10g, d11r, d11g;



  int N = fluide_stat->tailleX()-2;
  int M = fluide_stat->tailleY()-2;
	float hx = 1.0f/N;
	float hy = 1.0f/M;



	glBegin ( GL_QUADS );

		for ( i=0 ; i<=N ; i++ ) {
			x = (i-0.5f) *hx;
			for ( j=0 ; j<=M ; j++ ) {
				y = (j-0.5f)*hy;

				d00  =1055*fluide_stat->pressionLire(i,j) ;
				d01 = 1055*fluide_stat->pressionLire(i,j+1);
				d10 = 1055*fluide_stat->pressionLire(i+1,j);
				d11 = 1055*fluide_stat->pressionLire(i+1,j+1);
								/*astuce d'affichage, permet de rendre plus beau le champs*/
        d00r = 1-exp(-2*maxfloat(d00, 0.0f));
        d00g = 1-exp(-2*maxfloat(-d00, 0.0f));
        d01r = 1-exp(-2*maxfloat(d01, 0.0f));
        d01g = 1-exp(-2*maxfloat(-d01, 0.0f));
        d10r = 1-exp(-2*maxfloat(d10, 0.0f));
        d10g = 1-exp(-2*maxfloat(-d10, 0.0f));
        d11r = 1-exp(-2*maxfloat(d11, 0.0f));
        d11g = 1-exp(-2*maxfloat(-d11, 0.0f));

				glColor3f ( d00r ,
				            0.5f * d00g,d00g ); glVertex2f ( x, y );
				glColor3f ( d10r,
                    0.5f * d10g,d10g ); glVertex2f ( x+hx, y );
				glColor3f ( d11r,
                    0.5f * d11g,d11g ); glVertex2f ( x+hx, y+hy );
				glColor3f ( d01r,
                    0.5f *d01g,d01g ); glVertex2f ( x, y+hy );
			}
		}

	glEnd ();

}


static void dessineVortex ()
{
	int i, j;
	float x, y, d00, d01, d10, d11, d00r, d00g, d01r, d01g, d10r, d10g, d11r, d11g;



  int N = fluide_stat->tailleX()-2;
  int M = fluide_stat->tailleY()-2;

	float hx = 1.0f/N;
	float hy = 1.0f/M;

	glBegin ( GL_QUADS );

		for ( i=0 ; i<=N ; i++ ) {
			x = (i-0.5f) *hx;
			for ( j=0 ; j<=M ; j++ ) {
				y = (j-0.5f) *hy;

				d00  =fluide_stat->vortexLire(i,j) ;
				d01 = fluide_stat->vortexLire(i,j+1);
				d10 = fluide_stat->vortexLire(i+1,j);
				d11 =  fluide_stat->vortexLire(i+1,j+1);
				/*astuce d'affichage, permet de rendre plus beau le champs*/
        d00r = 1-exp(-2*maxfloat(d00, 0.0f));
        d00g = 1-exp(-2*maxfloat(-d00, 0.0f));
        d01r = 1-exp(-2*maxfloat(d01, 0.0f));
        d01g = 1-exp(-2*maxfloat(-d01, 0.0f));
        d10r = 1-exp(-2*maxfloat(d10, 0.0f));
        d10g = 1-exp(-2*maxfloat(-d10, 0.0f));
        d11r = 1-exp(-2*maxfloat(d11, 0.0f));
        d11g = 1-exp(-2*maxfloat(-d11, 0.0f));


				glColor3f ( d00r ,
				             d00g,0.5f *d00g ); glVertex2f ( x, y );
				glColor3f ( d10r,
                    d10g,0.5f *d10g ); glVertex2f ( x+hx, y );
				glColor3f ( d11r,
                     d11g,0.5f *d11g ); glVertex2f ( x+hx, y+hy );
				glColor3f ( d01r,
                    d01g,0.5f *d01g ); glVertex2f ( x, y+hy );
			}
		}

	glEnd ();
}


/*
  ----------------------------------------------------------------------
   relates mouse movements to forces sources
  ----------------------------------------------------------------------
*/

static void getMouvement ()
{
	int i, j;
  int N, M;


  N = fluide_stat->tailleX()-2;
  M = fluide_stat->tailleY()-2;

  //u_stat->annuler();

	if ( !mouse_down[0] && !mouse_down[2] ) return;

	i = (int)((       mx /(float)win_x)* N+1);
	j = (int)(((win_y-my)/(float)win_y)* M+1);
//u_stat->getX()->ecrire(10,12, 15.f);

	if ( i<1 || i>N || j<1 || j>M ) return;


	if ( mouse_down[0] ) {
	//  cout<<u_stat->getX()->lire(i,j);
   fluide_stat->vitesseEcrire(i,j, forces * (mx-omx), forces* (omy-my));
//cout<<u_stat->getX()->lire(i,j);

	}

	if ( mouse_down[2] ) {
		fluide_stat->diffusionEcrire(i,j, 200.0);
	}

	omx = mx;
	omy = my;

	return;
}



static void affichageCommence ( void )
{
	glViewport ( 0, 0, win_x, win_y );
	glMatrixMode ( GL_PROJECTION );
	glLoadIdentity ();
	gluOrtho2D ( 0.0, 1.0, 0.0, 1.0 );
	glClearColor ( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear ( GL_COLOR_BUFFER_BIT );
}

static void affichageTermine ( void )
{
	glutSwapBuffers ();
}





static void floglutClavier (unsigned char key, int x, int y )
{

	switch ( key )
	{
		case 'c':
		case 'C':
			fluide_stat->vitesseReinitialiser();
			fluide_stat->diffusionReinitialiser();


			break;

		case 'q':
		case 'Q':
			exit ( 0 );
			break;

		case 'v':
		case 'V':
		  whatdraw = VITESSE;
			break;
		case 'd':
		case 'D':
		  whatdraw = DIFFUSION;
			break;
 		case 'r':
		case 'R':
		  whatdraw = VORTEX;
			break;

    case 'p':
    case 'P':
      whatdraw = PRESSION;
      break;

	}
}

static void floglutMouse ( int button, int state, int x, int y )
{
	omx = mx = x;
	omx = my = y;

	mouse_down[button] = state == GLUT_DOWN;
}

static void floglutMouvementSouris ( int x, int y )
{
	mx = x;
	my = y;
}

static void floglutRedimensionnement ( int width, int height )
{
	glutSetWindow ( win_id );
	glutReshapeWindow ( width, height );

	win_x = width;
	win_y = height;
}






static void floglutPrincipal ()
{
	getMouvement ();

//for(i=30; i<=30;i++)
//{
 // u_stat->getX()->ecrire(10,40, 0.3f);
//}

  //u_stat->getX()->ecrire(10,43, 0.3f);


//u_stat->getX()->ecrire(10,54, 0.1f);

  fluide_stat->evoluerVitesse();
  fluide_stat->evoluerDiffusion();

  if(whatdraw == VORTEX)
  fluide_stat->calculVortex();

  if(whatdraw == PRESSION)
    fluide_stat->calculPression();


	glutSetWindow ( win_id );
	glutPostRedisplay ();
}

static void floglutAffiche ()
{
	affichageCommence();
  switch(whatdraw)
   { case VITESSE:
    	 dessineVitesse ();
      break;
    case DIFFUSION:
       dessineDiffusion();
       break;
    case VORTEX:
       dessineVortex();
       break;
    case PRESSION:
       dessinePression();
       break;
    default:
       dessineVitesse();
       break;
   }

	affichageTermine ();
}


/*
  ----------------------------------------------------------------------
   open_glut_window --- open a glut compatible window and set callbacks
  ----------------------------------------------------------------------
*/

void open_glut_window ( Fluide* fluide, float force)
{
  fluide_stat = fluide;


  forces =force;

	glutInitDisplayMode ( GLUT_RGBA | GLUT_DOUBLE);

	glutInitWindowPosition ( 0, 0 );
	glutInitWindowSize ( win_x, win_y );
	win_id = glutCreateWindow ( "Navier-Stokes, neoflo" );

	glClearColor ( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear ( GL_COLOR_BUFFER_BIT );
	glutSwapBuffers ();
	glClear ( GL_COLOR_BUFFER_BIT );
	glutSwapBuffers ();

//	affichageCommence();

	glutKeyboardFunc ( floglutClavier );
	glutMouseFunc ( floglutMouse );
	glutMotionFunc ( floglutMouvementSouris );
	glutReshapeFunc ( floglutRedimensionnement );
	glutIdleFunc ( floglutPrincipal );
	glutDisplayFunc ( floglutAffiche );
}


