#include <iostream>
#include <vector>


#include "champsscalaires.h"
#include "champsvecteurs.h"

#include "navier.h"





using namespace std;


struct diffTriplet
{
 int x;
 int y;
 float value;

};

Fluide::Fluide(int n, int m, float dtin)
{
  vitesse = new ChampsVecteurs(n,m);
  pression = new ChampsScalaires(n,m);
  diffusion = new ChampsScalaires(n,m);

  vect_tempo1 = new ChampsVecteurs(n,m);
  vect_tempo2 = new ChampsVecteurs(n,m);

  divpp = new ChampsScalaires(n,m);
  phi = new ChampsScalaires(n,m);
  vortex = new ChampsScalaires(n,m);

  murhaut = false;
  murgauche = false;
  murdroit = false;
  murbas = true;

  forcefumeex = 0.0f;
  forcefumeey = 0.0f;


  dt = dtin;

}

Fluide::~Fluide()
{
 delete vitesse;
 delete pression;
 delete diffusion;
 delete vect_tempo1;
 delete vect_tempo2;
 delete divpp;
 delete phi;
 delete vortex;


/*destruction de la liste d'obstacle*/
 obstacleDetruire();

}

int Fluide::tailleX() const
{
 return vitesse->tailleX();
}

int Fluide::tailleY() const
{
 return vitesse->tailleY();
}




float Fluide::diffusionLire(int i, int j) const
{
 return diffusion->lire(i,j);
}

float Fluide::pressionLire(int i, int j) const
{
 return pression->lire(i,j);
}

float Fluide::vortexLire(int i, int j) const
{
 return vortex->lire(i,j);
}

void Fluide::diffusionReinitialiser(void)
{
 diffusion->annuler();
}

void Fluide::diffusionEcrire(int i, int j, float valeur)
{
 if(i<1)
  i =1;

 if(j<1)
  j= 1;
 if(j>=tailleY()-2)
  j = tailleY()-3;

 if(i>=tailleX()-2) i = tailleX()-3;

 diffusion->ecrire(i, j, valeur);
}

void Fluide::diffusionAutomatique(int x, int y, float valeur)
{
  diffTriplet triplet;
  triplet.x =x;
  triplet.y = y;
  triplet.value = valeur;

  listeDiffusion.push_back(triplet);

}

void Fluide::pointdiffusionEcrire()
{
  std::list<diffTriplet>::iterator diff;
  diffTriplet triplet;


for (diff=listeDiffusion.begin(); diff!=listeDiffusion.end(); diff++)
{
  triplet = *diff;
  diffusionEcrire(triplet.x, triplet.y, triplet.value);
}

}

void Fluide::diffusionAutomatiqueDetruire()
{
  listeDiffusion.clear();
}


void Fluide::setMurGauche()
{
 murhaut = true;
}

void Fluide::setMurDroit()
{
  murdroit = true;
}

void Fluide::setMurHaut()
{
  murhaut = true;
}

void Fluide::setMurBas()
{
  murbas = true;
}

void Fluide::unsetMurGauche()
{
  murgauche = false;
}


void Fluide::unsetMurDroit()
{
  murdroit = false;
}

void Fluide::unsetMurHaut()
{
 murhaut = false;
}


void Fluide::unsetMurBas()
{
 murbas = false;
}



void Fluide::ajouterObstacle(int x, int y, int w, int h)
{



  if (x <= 0)
   x = 1;

  if(y<=0)
   y = 1;

if(w+x > vitesse->tailleX())
  w = vitesse->tailleX()-x-3;
if(h+y > vitesse->tailleY())
  h = vitesse->tailleY()-y-3;



Obstacle* obsajj = new Obstacle(x,y,w,h);

lobs.push_back(obsajj);

}

void Fluide::obstacleDetruire()
{
 ListeObstacle* liste = &this->lobs;
 ListeObstacle::iterator obs;

for (obs=liste->begin(); obs!=liste->end(); obs++)
{
  delete *obs;
}

liste->clear();

}


/*out dtruit*/
void Fluide::calculAdvectionV(ChampsVecteurs* in, ChampsVecteurs* out)
{
  float xd, yd;
  int i,j;
  int n = in->tailleX()-2;
  int m = in->tailleY()-2;
  float pas = in->pas();
  ChampsScalaires* ux = in->getX();
  ChampsScalaires* uy = in->getY();
  ChampsScalaires* outx = out->getX();
  ChampsScalaires* outy = out->getY();

  int id, jd;
  float Asw, Ase, Ane, Anw;

  float dtspas = this->dt / pas;

  for(j=1;j<=m;j++)
    {
    for(i=1;i<=n;i++)
      {

        xd = (float) i - dtspas * ux->lire(i,j);

        yd = (float) j - dtspas * uy->lire(i,j);
        /*rebascule dans les bords*/
        if(xd < 0.5f) xd = 0.5f;
        if(yd < 0.5f) yd = 0.5f;
        if(xd > n+0.5f) xd = n+0.5f;
        if(yd > m+0.5f) yd = m+0.5f;
        id = (int) xd;


        jd = (int) yd;
        Asw = (1.0f+id-xd) * (1.0f+jd-yd);
        Ase = (xd - id) * (1.0f+jd-yd);
        Anw = (1.0f+id-xd)  * (yd - jd);
        Ane = (xd - id) * (yd - jd);
        outx->ecrire(i,j,
          Asw * ux->lire(id, jd) +
          Ase * ux->lire(id+1,jd) +
          Anw * ux->lire(id, jd+1) +
          Ane * ux->lire(id+1, jd+1));
        outy->ecrire(i,j,
          Asw * uy->lire(id, jd) +
          Ase * uy->lire(id+1,jd) +
          Anw * uy->lire(id, jd+1) +
          Ane * uy->lire(id+1, jd+1));

      }
    }

 forcerBords(out);
}




/********************
   out dtruit
********************/
void Fluide::div_pascarreChamps(ChampsVecteurs *u, ChampsScalaires* out)
{
  int i,j;
  int n = u->tailleX()-2;
  int m = u->tailleY()-2;
  float pas = u->pas();
  ChampsScalaires* ux = u->getX();
  ChampsScalaires* uy = u->getY();

  for(j=1;j<=m;j++)
    {
    for(i=1;i<=n;i++)
     {
      out->ecrire(i,j,
        0.5 * (ux->lire(i+1,j) - ux->lire(i-1,j) + uy->lire(i, j+1) - uy->lire(i, j-1)) * pas);
     }
    }

  forcerBordsScalaires(out);
}



void Fluide::calculProjection(ChampsVecteurs *u, ChampsScalaires* divpp, ChampsScalaires* phi)
{
 int n;
 int m;
 int i,j, k;
 n = u->tailleX()-2;
 m = u->tailleY()-2;
 ChampsScalaires* ux = u->getX();
 ChampsScalaires* uy = u->getY();
 float pas = u->pas();


 div_pascarreChamps(u, divpp);
 phi->annuler();

 /*resolution*/
 for(k=1;k<20;k++)
   {
    for(j=1;j<=m;j++)
     {
      for(i=1;i<=n;i++)
        {
         phi->ecrire(i, j,
           (-divpp->lire(i,j) +
           ( phi->lire(i+1,j) + phi->lire(i-1,j) + phi->lire(i,j+1) + phi->lire(i,j-1))) /4.0f);
        }
     }
   forcerBordsScalaires(phi);

   }

 /*soustrait la divergence*/
    for(j=1;j<=m;j++)
     {
      for(i=1;i<=n;i++)
        {
           ux->somme(i,j, -0.5 * (phi->lire(i+1,j) - phi->lire(i-1,j)  ) /pas );
           uy->somme(i,j, -0.5 * (phi->lire(i,j+1) - phi->lire(i,j-1)  ) /pas );
        }
     }
}






struct ModificationVitesse
{
   int i;
   int j;
   float ix;
   float jy;
};

void Fluide::vitesseEcrire(int i, int j, float ix, float jy)
{
  ModificationVitesse aajouter;

aajouter.i =i;
aajouter.j = j;
aajouter.ix = ix;
aajouter.jy = jy;

lmodif.push(aajouter);
}

std::pair<float, float> Fluide::vitesseLire(int i, int j) const
{
  return(std::make_pair(vitesse->getX()->lire(i,j), vitesse->getY()->lire(i,j)));
}

void Fluide::vitesseReinitialiser()
{
   vitesse->getX()->annuler();
   vitesse->getY()->annuler();
}

void Fluide::modifierChampsVitesse(ChampsVecteurs* u)
{
 ModificationVitesse aretirer;

    while(!lmodif.empty())
    {
       aretirer = lmodif.top();
       lmodif.pop();
       u->getX()->ecrire(aretirer.i, aretirer.j, aretirer.ix);
       u->getY()->ecrire(aretirer.i, aretirer.j, aretirer.jy);
    }
}


void Fluide::ecrireForceFumee(float x, float y)
{
 forcefumeex =x;
 forcefumeey = y;
}


void Fluide::evoluerFumee(ChampsVecteurs* u, ChampsScalaires* diff)
{
 /*resolution*/

if(forcefumeex!=0.0f)
{
 u->getX()->sommeScalaires(diff, forcefumeex);
}

if(forcefumeey!=0.0f)
{
  u->getY()->sommeScalaires(diff, forcefumeey);
}




}


void Fluide::evoluerVitesse()
{
 ChampsVecteurs* permute;
 //calculViscositeV(u, &tempo1, nu, dt);

 calculAdvectionV(vitesse, vect_tempo2);

 /*astuce de sioux pour ne pas appeler calculProjection deux fois*/

 evoluerFumee(vect_tempo2, diffusion );

modifierChampsVitesse(vect_tempo2);


 calculProjection(vect_tempo2, divpp, phi);


/*permutation*/
 permute = vitesse;
 vitesse = vect_tempo2;
 vect_tempo2 = permute;
}




/**********************************************/
/**                                          **/
/**                                          **/
/**                                          **/
/**        Calcul de pression                **/
/**                                          **/
/**                                          **/
/**********************************************/


void Fluide::calculUNablaU(ChampsVecteurs* in, ChampsVecteurs* out)
{
  int n = in->tailleX()-2;
  int m = out->tailleY()-2;
  int i, j;
  ChampsScalaires* ux = in->getX();
  ChampsScalaires* uy = in->getY();
  ChampsScalaires* outx = out->getX();
  ChampsScalaires* outy = out->getY();
  float h = in->pas();

  for(j=1; j<=m;j++)
   for(i=1; i<=n;i++)
   {
       outx->ecrire(i,j,
         ux->lire(i,j) * (ux->lire(i+1,j) - ux->lire(i-1,j)) / (2*h)
        +uy->lire(i,j) * (ux->lire(i,j+1) - ux->lire(i,j-1)) / (2*h));
      outy->ecrire(i,j,
         ux->lire(i,j) * (uy->lire(i+1,j) - uy->lire(i-1,j)) / (2*h)
        +uy->lire(i,j) * (uy->lire(i,j+1) - uy->lire(i,j-1)) / (2*h));
   }

  forcerBords(out);

}


void Fluide::calculPression()
{
  int n = vitesse->tailleX()-2;
  int m = vitesse->tailleY()-2;
  int k,i,j;


  calculUNablaU(vitesse, vect_tempo1);
  div_pascarreChamps(vect_tempo1, divpp);


 pression->annuler();
 /*resolution*/
 for(k=1;k<20;k++)
   {
    for(j=1;j<=m;j++)
     {
      for(i=1;i<=n;i++)
        {
         pression->ecrire(i, j,
           (-divpp->lire(i,j) +
           ( pression->lire(i+1,j) + pression->lire(i-1,j) + pression->lire(i,j+1) + pression->lire(i,j-1))) /4.0f);
        }
     }
   forcerBordsScalaires(pression);

   }
}

/**********************************************/
/**                                          **/
/**                                          **/
/**                                          **/
/**        Calcul de diffusion               **/
/**                                          **/
/**                                          **/
/**********************************************/




void Fluide::calculAdvectionS(ChampsScalaires* in, ChampsScalaires* out, ChampsVecteurs* u)
{

  float xd, yd;
  int i,j;
  int n = u->tailleX()-2;
  int m = u->tailleY()-2;
  float pas = u->pas();
  ChampsScalaires* ux = u->getX();
  ChampsScalaires* uy = u->getY();


  int id, jd;
  float Asw, Ase, Ane, Anw;

  float dtspas = this->dt / pas;

  for(j=1;j<=m;j++)
    {
    for(i=1;i<=n;i++)
      {
        xd = (float) i -dtspas * ux->lire(i,j);
        yd = (float) j - dtspas * uy->lire(i,j);
        /*rebascule dans les bords*/
        if(xd < 0.5f) xd = 0.5f;
        if(yd < 0.5f) yd = 0.5f;
        if(xd > n+0.5f) xd = n+0.5f;
        if(yd > m+0.5f) yd = m+0.5f;
        id = (int) xd;
        jd = (int) yd;
        Asw = (1.0f+id-xd) * (1.0f+jd-yd);
        Ase = (xd - id) * (1.0f+jd-yd);
        Anw = (1.0f+id-xd)  * (yd - jd);
        Ane = (xd - id) * (yd - jd);
        out->ecrire(i,j,
          Asw * in->lire(id, jd) +
          Ase * in->lire(id+1,jd) +
          Anw * in->lire(id, jd+1) +
          Ane * in->lire(id+1, jd+1));
      }
    }

 forcerBordsScalaires(out);
}



void Fluide::evoluerDiffusion()
{
 ChampsScalaires* permute;


 pointdiffusionEcrire();

 calculAdvectionS(diffusion, phi, vitesse);

 permute = diffusion;
 diffusion  = phi;
 phi = permute;




}




/******************************************/
/**** Calcul Vortex                      **/
/******************************************/
void Fluide::calculVortex()
{
int n = vitesse->tailleX()-2;
 int m = vitesse->tailleY()-2;
 int i,j;
 ChampsScalaires* ux = vitesse->getX();
 ChampsScalaires* uy = vitesse->getY();

  float h = vitesse->pas();
 for(j=1;j<=m;j++)
  for(i=1;i<=n;i++)
 {
   vortex->ecrire(i,j,
 (uy->lire(i+1,j) - uy->lire(i-1,j)) / (2*h)
  - (ux->lire(i,j+1) - ux->lire(i,j-1)) / (2*h));
 }

 forcerBordsScalaires(vortex);
}




/***********************************
 *
 * Calcul des conditions aux bords
 *
 *
 ************************************/



void Fluide::forcerBordsXObstacle(ChampsScalaires* ux, Obstacle* obs)
{
int i;
 int j;
 unsigned int n;
 unsigned int m;

 n= ux->tailleX()-2;
 m = ux->tailleY()-2;

 int squarey = obs->getY();
int squarex = obs->getX();
int squareh = obs->getH();
int squarew = obs->getW();


for(j=squarey;j<=(squarey + squareh);j++)
  {
    ux->ecrire(squarex-1, j, 0.0f);
    ux->ecrire(squarex + squarew+1, j, 0.0f);
    for(i=squarex;i<=(squarex+squarew);i++)
      ux->ecrire(i, j , 0.0f);
  }



for(i=squarex;i<=(squarex+squarew);i++)
{
ux->ecrire(i, squarey-1, ux->lire(i, squarey-2));
ux->ecrire(i, squarey+squareh+1, ux->lire(i, squarey+squareh+2));

}

ux->ecrire(squarex-1,squarey-1, 0.5 * (ux->lire(squarex, squarey-1) + ux->lire(squarex-1,squarey)));
ux->ecrire(squarex+squarew+1,squarey-1, 0.5 * (ux->lire(squarex+squarew+1,squarey) +
                        ux->lire(squarex+squarew,squarey-1)));
ux->ecrire(squarex-1,squarey+squareh+1, 0.5 * (ux->lire(squarex,squarey+squareh+1) + ux->lire(squarex-1,
                squarey+squareh)));
ux->ecrire(squarex+squarew+1,squarey+squareh+1,
               0.5 * (ux->lire(squarex+squarew,squarey+squareh+1) + ux->lire(squarex+squarew+1,squarey+squareh)));


}

void Fluide::forcerBordsYObstacle(ChampsScalaires *uy, Obstacle* obs)
{
int i;
 int j;
 unsigned int n;
 unsigned int m;

 n= uy->tailleX()-2;
 m = uy->tailleY()-2;

 int squarey = obs->getY();
int squarex = obs->getX();
int squareh = obs->getH();
int squarew = obs->getW();



for(i=squarex;i<=(squarex+squarew);i++)
  {
    uy->ecrire(i, squarey-1, 0.0f);
    uy->ecrire(i, squarey+squarew+1, 0.0f);
    for(j=squarey; j<=(squarey+squareh); j++)
      uy->ecrire(i, j, 0.0f);

  }

for(j=squarey; j<=(squarey+squareh);j++)
{
uy->ecrire(squarex-1,j, uy->lire(squarex-2,j));
uy->ecrire(squarex+squarew+1,j , uy->lire(squarex+squarew+2, j));

}

uy->ecrire(squarex-1,squarey-1, 0.5 * (uy->lire(squarex, squarey-1) + uy->lire(squarex-1,squarey)));
uy->ecrire(squarex+squarew+1,squarey-1, 0.5 * (uy->lire(squarex+squarew+1,squarey) +
                        uy->lire(squarex+squarew,squarey-1)));
uy->ecrire(squarex-1,squarey+squareh+1, 0.5 * (uy->lire(squarex,squarey+squareh+1) + uy->lire(squarex-1,
                squarey+squareh)));
uy->ecrire(squarex+squarew+1,squarey+squareh+1,
               0.5 * (uy->lire(squarex+squarew,squarey+squareh+1) + uy->lire(squarex+squarew+1,squarey+squareh)));

}



void Fluide::forcerBordsXListeObstacle(ChampsScalaires*ux)
{

 ListeObstacle* liste = &this->lobs;
ListeObstacle::iterator obs;

for (obs=liste->begin(); obs!=liste->end(); obs++)
 {
  forcerBordsXObstacle(ux, *obs);
 }


}

void Fluide::forcerBordsYListeObstacle(ChampsScalaires*uy)
{

 ListeObstacle* liste = &this->lobs;
 ListeObstacle::iterator obs;

for (obs=liste->begin(); obs!=liste->end(); obs++)
 {
  forcerBordsYObstacle(uy, *obs);
 }



}


void Fluide::forcerBordsScalairesObstacle(ChampsScalaires* in, Obstacle* obs)
{
 int i;
 int j;
 int n,m;

 n = in->tailleX()-2;
 m = in->tailleY()-2;
  int squarey = obs->getY();
int squarex = obs->getX();
int squareh = obs->getH();
int squarew = obs->getW();

for(j=squarey; j<=squarey+squareh;j++)
{
 in->ecrire(squarex-1, j,
    in->lire(squarex-2,j));
 in->ecrire(squarex+squarew+1,j,
     in->lire(squarex+squarew+2, j));
     for(i=squarex; i<=squarex+squarew;i++)
       in->ecrire(i,j, 0.0f);

}

for(i=squarex; i<=squarex+squarew;i++)
{



 in->ecrire(i, squarey-1,
    in->lire(i,squarey-2));
 in->ecrire(i,squarey+squareh+1,
     in->lire(i, squarey+squareh+2));

}
in->ecrire(squarex-1,squarey-1, 0.5 * (in->lire(squarex, squarey-1) + in->lire(squarex-1,squarey)));
in->ecrire(squarex+squarew+1,squarey-1, 0.5 * (in->lire(squarex+squarew+1,squarey) +
                        in->lire(squarex+squarew,squarey-1)));
in->ecrire(squarex-1,squarey+squareh+1, 0.5 * (in->lire(squarex,squarey+squareh+1) + in->lire(squarex-1,
                squarey+squareh)));
in->ecrire(squarex+squarew+1,squarey+squareh+1,
               0.5 * (in->lire(squarex+squarew,squarey+squareh+1) + in->lire(squarex+squarew+1,squarey+squareh)));



}

void Fluide::forcerBordsScalairesListeObstacle(ChampsScalaires* in)
{
// Obstacle * obs;
 ListeObstacle* liste = &this->lobs;


for (ListeObstacle::iterator obs=liste->begin(); obs!=liste->end(); obs++)
 {
  forcerBordsScalairesObstacle(in, *obs);
 }


}





void Fluide::forcerBordsX(ChampsScalaires* ux)
{
   int i;
 int j;
 int n;
 int m;

 n= ux->tailleX()-2;
 m = ux->tailleY()-2;


if(murgauche)
{
  for(j=1;j<=m;j++)
  {
    ux->ecrire(0,j, 0.0f);
    ux->ecrire(n+1,j, 0.0f);
  }
ux->ecrire(0,0,     0.5*(ux->lire(0,1)   + ux->lire(1,0)));
ux->ecrire(0,m+1,   0.5*(ux->lire(1,m+1) + ux->lire(0,m)));
}

if(murdroit)
{
 for(i=1;i<=n;i++)
 {
   ux->ecrire(i, 0, ux->lire(i,1));
   ux->ecrire(i, m+1, ux->lire(i, m+1));
 }


ux->ecrire(n+1,m+1, 0.5*(ux->lire(n,m+1) + ux->lire(n+1,m)));
ux->ecrire(n+1,0,   0.5*(ux->lire(n,0)   + ux->lire(n+1,1)));
}


forcerBordsXListeObstacle(ux);
}

void Fluide::forcerBordsY(ChampsScalaires* uy)
{
 int i;
 int j;
 int n;
 int m;

 n = uy->tailleX()-2;
 m = uy->tailleY()-2;

 if(murbas)
 {
 for(j=1;j<=m;j++)
 {
   uy->ecrire(0,j, uy->lire(1,j));
   uy->ecrire(n+1,j, uy->lire(n,j));
 }
uy->ecrire(0,0, 0.5*(uy->lire(0,1) + uy->lire(1,0)));
uy->ecrire(0,m+1, 0.5*(uy->lire(1,m+1) + uy->lire(0,m)));

 }

 if(murhaut)
 {
      for(i=1;i<=n;i++)
 {

   uy->ecrire(i, 0, 0.0f);
   uy->ecrire(i, m+1, 0.0f);
 }


uy->ecrire(n+1,m+1, 0.5*(uy->lire(n,m+1) + uy->lire(n+1,m)));

uy->ecrire(n+1,0, 0.5*(uy->lire(n,0) + uy->lire(n+1,1)));


 }


forcerBordsYListeObstacle(uy);
}

void Fluide::forcerBords(ChampsVecteurs* u)
{
 forcerBordsX(u->getX());
 forcerBordsY(u->getY());
}


/* in dtruit*/
void Fluide::forcerBordsScalaires(ChampsScalaires* in)
{

 int i;
 int j;
 int n,m;

 n = in->tailleX()-2;
 m = in->tailleY()-2;

if(this->murbas)
{
  for(i=1;i<=n;i++)
  {
   in->ecrire(i, 0,
     in->lire(i,1));
  }
  in->ecrire(0,0,
    0.5 * (in->lire(0,1) + in->lire(1,0)));
}

if(this->murhaut)
{
    for(i=1;i<=n;i++)
  {
   in->ecrire(i, m+1,
     in->lire(i,m));
  }

}


if(this->murgauche)
{
 for(j=1;j<=m;j++)
  {
   in->ecrire(0, j,
     in->lire(1,j));
  }
}

if(this->murdroit)
{
  for(j=1;j<=m;j++)
  {
     in->ecrire(n+1, j,
     in->lire(n,j));
  }

}

  in->ecrire(0,0,
    0.5 * (in->lire(0,1) + in->lire(1,0)));
  in->ecrire(n+1,0,
    0.5 * (in->lire(n+1,1) + in->lire(n,0)));
  in->ecrire(0,m+1,
    0.5 * (in->lire(1,m+1) + in->lire(1,m)));
  in->ecrire(n+1,m+1,
    0.5 * (in->lire(n+1,0) + in->lire(n,1)));


 this->forcerBordsScalairesListeObstacle(in);

}

