#ifndef _NAVIER_H
#define _NAVIER_H


#include <iostream>
#include <vector>
#include <list>
#include <stack>

#include "obstacle.h"



/*
 Title: Module gestion de fluides
 */

/*
 Topic: Document du module sur la gestion des fluides, ce module permet
 la cration d'un fluide, l'ajout d'obstacle et son volution
 */

struct ModificationVitesse;

typedef std::list<Obstacle*>  ListeObstacle;
typedef std::vector<ModificationVitesse> ListeModification;
struct diffTriplet;

/*
 class: Fluide
 Type abstrait permettant la gestion des fluides
*/
class Fluide
{
   private:
    ChampsVecteurs* vitesse;
    ChampsScalaires* diffusion;
    ChampsScalaires* pression;

/* variable temporaire, peut tre dtruite n'importe quand*/
    ChampsVecteurs* vect_tempo1;
    ChampsVecteurs* vect_tempo2;
    ChampsScalaires* divpp;
    ChampsScalaires* phi;
    ChampsScalaires* vortex;

/* liste obstacle*/
    ListeObstacle lobs;
    std::stack<ModificationVitesse> lmodif;
    std::stack<Obstacle> pileobs;
    std::list<diffTriplet> listeDiffusion;

/*caractristique du fluide*/

    float dt;

    float forcefumeex;
    float forcefumeey;

    bool murhaut;
    bool murbas;
    bool murgauche;
    bool murdroit;

    void modifierChampsVitesse(ChampsVecteurs* u);

    /*mthode prive de calcul d'advection/obstacle/bords*/
    void calculAdvectionV(ChampsVecteurs* in, ChampsVecteurs* out);
    void calculAdvectionS(ChampsScalaires* in, ChampsScalaires* out, ChampsVecteurs* u);
    void calculUNablaU(ChampsVecteurs* in, ChampsVecteurs* out);
    void calculProjection(ChampsVecteurs *u, ChampsScalaires* divpp, ChampsScalaires* phi);
    void div_pascarreChamps(ChampsVecteurs *u, ChampsScalaires* out);



    void forcerBordsXObstacle(ChampsScalaires* ux, Obstacle* obs);
    void forcerBordsYObstacle(ChampsScalaires *uy, Obstacle* obs);
    void forcerBordsXListeObstacle(ChampsScalaires*ux);
    void forcerBordsYListeObstacle(ChampsScalaires*uy);
    void forcerBordsScalairesObstacle(ChampsScalaires* in, Obstacle* obs);
    void forcerBordsScalairesListeObstacle(ChampsScalaires*in);
    void forcerBordsX(ChampsScalaires* ux);
    void forcerBordsY(ChampsScalaires* uy);
    void forcerBords(ChampsVecteurs* u);
    void forcerBordsScalaires(ChampsScalaires* in);


   /*fume*/
      void evoluerFumee(ChampsVecteurs* u, ChampsScalaires* diff);

   /*diffusion automatique*/
   void pointdiffusionEcrire();

   public:


    /*
     Constructor: Fluide

      Paramtres:
        n - taille horizontale sans les bords
        m - taille verticale sans les bords
        dtin - pas de discrtisation temporelle
        rhoin - coefficient de pression (pas important car souvent non utilis
     Prcondition:
       n et m suprieur  3 au moins.
    */
    Fluide(int n, int m,  float dtin);
    /*
     Destructor: ~Fluide
      Le destroyeur de classe
    */
    ~Fluide();

     int tailleX() const;
     int tailleY() const;

    /*
     Method: ajouterObstacle
      Permet l'ajout d'un obtacle

      Paramtres:
        obs - l'obstacle  ajouter, en cas de dpassement par rapport au fluide, l'obstacle sera redimensionner
     */
    void ajouterObstacle(int x, int y, int w, int h);



    /*
     Method: evoluerVitesse
      Fait voluer la vitesse sur le pas de temps dtin

           Attention: Ce champs est susceptible de changer lors de l'appel de evoluerVitesse()
     ou de calculDiffusion()
     */
    void evoluerVitesse();

    /*
     Method: calculPression
      Inscrit dans le champs recuprable par la mthode getPressiion() la pression
       cet instant
    */
    void calculPression();

    /*
     Method: calculDiffusion
      Calcul le champs de diffusion, attention, si l'on souhaite utiliser le champs de diffusion
      cette mthode doit tre appel  chaque fois que la mthode evoluerVitesse est appele
    */
    void evoluerDiffusion();

    /*
     Method: calculVortex
      Calcul dans le champs recuprable par la mthode getVortex()
       la dtection de vortex
    */
    void calculVortex();

   /*
    Method: modifierVitesse
     Doit tre obligatoirement appel si l'on souhaite modifier le vecteur vitesse
     ( cause d'optimisations)

     Paramtres:
      i - position horizontale que l'on souhaite modifier
      j - position verticale
      ix - valeur selon l'axe x
      jy - valeur selon l'axe y
    */
    void vitesseEcrire(int i, int j, float ix, float jy);

    std::pair<float, float> vitesseLire(int i, int j) const;
    void vitesseReinitialiser();

    float diffusionLire(int i, int j) const;
    float pressionLire(int i, int j) const;
    float vortexLire(int i, int j) const;

    void diffusionEcrire(int i, int j, float valeur);
    void diffusionReinitialiser(void);

    void diffusionAjouterConstant(int x, int y, float valeur);

    void setMurGauche();
    void setMurDroit();
    void setMurHaut();
    void setMurBas();
    void unsetMurGauche();
    void unsetMurDroit();
    void unsetMurHaut();
    void unsetMurBas();


   /*fume*/
   void ecrireForceFumee(float x, float y);
   void diffusionAutomatique(int x, int y, float valeur);
   void diffusionAutomatiqueDetruire();

   void obstacleDetruire();

};


#endif
